package ai.pai.lensman.session;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.os.IBinder;
import android.util.Log;

import java.util.ArrayList;

import ai.pai.lensman.App;
import ai.pai.lensman.bean.PhotoBean;
import ai.pai.lensman.bean.SessionBean;
import ai.pai.lensman.db.DBService;
import ai.pai.lensman.service.UploadService;
import android.content.ServiceConnection;
import com.gprinter.aidl.GpService;



public class SessionPresenter implements SessionContract.Presenter, SessionInteractor.SessionListener {

    private SessionInteractor interactor;
    private ArrayList<PhotoBean> photoList;
    private SessionContract.View sessionView;
    private SessionBean sessionBean;
    private boolean isWorking;

    private GpService mGpService = null;
    private PrinterServiceConnection conn = null;



    public SessionPresenter(SessionBean sessionBean, SessionContract.View view) {
        this.sessionView = view;
        this.sessionBean = sessionBean;
        interactor = new SessionInteractor(sessionBean, this);
    }

    @Override
    public void start() {
        photoList = DBService.getInstance().getPhotoListBySessionId(sessionBean.sessionId);
        if (photoList.size() == 0) {
            sessionView.showEmptyView();
        } else {
            sessionView.showPhotoRecyclerView();
            for (PhotoBean photoBean : photoList) {
                sessionView.addNewPhoto(photoBean);
            }
        }
        interactor.startSession();
        connection();
    }

    @Override
    public void stop() {
        interactor.endSession();
        isWorking = false;
        if (conn != null) {
            App.getAppContext().unbindService(conn);
        }
    }

    @Override
    public void onSessionStartSuccess(String session) {
        isWorking = true;
        sessionView.showToast("session启动成功");
    }

    @Override
    public void onSessionStartError(String session) {
        sessionView.showToast("session启动失败,自动重试中");
        interactor.startSession();
    }

    @Override
    public void onSessionPhotoCaptured(final PhotoBean bean) {
        if(!isWorking){
            return;
        }
        DBService.getInstance().addPhotoBean(bean);
        sessionView.showPhotoRecyclerView();
        sessionView.addNewPhoto(bean);
        App.getAppContext().startService(new Intent(App.getAppContext(), UploadService.class));
    }

    @Override
    public void onSessionEnd(String session) {

    }

    @Override
    public void swipeToDeletePhoto(int index) {
        PhotoBean photoBean = photoList.get(index);
        photoList.remove(photoBean);
        DBService.getInstance().deletePhotoBean(photoBean);
        interactor.deletePhoto(photoBean);
    }

    private void connection() {
        conn = new PrinterServiceConnection();
        Intent intent = new Intent("com.gprinter.aidl.GpPrintService");
        App.getAppContext().bindService(intent, conn, Context.BIND_AUTO_CREATE);
    }

    class PrinterServiceConnection implements ServiceConnection {
        @Override
        public void onServiceDisconnected(ComponentName name) {
            Log.i("ServiceConnection", "onServiceDisconnected() called");
            mGpService = null;
        }

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            mGpService = GpService.Stub.asInterface(service);
        }
    }
}
